@@ -5,17 +5,21 @@ Do a little skipping about with types to demonstrate its usage.
 """
 import json
 import sys
-from datetime import datetime
+from datetime import date, datetime
 from typing import Any, Dict, Generic, List, Optional, TypeVar
 
-from pydantic import BaseModel, NoneStr, StrictBool, root_validator, validator
+from pydantic import BaseModel, NoneStr, PyObject, StrictBool, root_validator, validate_arguments, validator
 from pydantic.fields import Field
 from pydantic.generics import GenericModel
+from pydantic.typing import ForwardRef
 
 
 class Flags(BaseModel):
     strict_bool: StrictBool = False
 
+    def __str__(self) -> str:
+        return f'flag={self.strict_bool}'
+
 
 class Model(BaseModel):
     age: int
@@ -33,7 +37,7 @@ class Model(BaseModel):
     def root_check(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         return values
 
-    @root_validator(pre=True)
+    @root_validator(pre=True, allow_reuse=False, skip_on_failure=False)
     def pre_root_check(cls, values: Dict[str, Any]) -> Dict[str, Any]:
         return values
 
@@ -115,3 +119,29 @@ if sys.version_info >= (3, 7):
 class WithField(BaseModel):
     age: int
     first_name: str = Field('John', const=True)
+
+
+@validate_arguments
+def foo(a: int, *, c: str = 'x') -> str:
+    return c * a
+
+
+foo(1, c='thing')
+foo(1)
+
+
+class Foo(BaseModel):
+    a: int
+
+
+FooRef = ForwardRef('Foo')
+
+
+class MyConf(BaseModel):
+    str_pyobject: PyObject = Field('datetime.date')
+    callable_pyobject: PyObject = Field(date)
+
+
+conf = MyConf()
+var1: date = conf.str_pyobject(2020, 12, 20)
+var2: date = conf.callable_pyobject(2111, 1, 1)
